// Copyright 2015 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

//go:generate go run gen.go

// Package colornames provides named colors as defined in the SVG 1.1 spec.
//
// See https://www.w3.org/TR/SVG11/types.html#ColorKeywords
package colornames
